@@ -14,18 +14,26 @@
 #  limitations under the License.
 #
 
-import sys
 import dis
-from typing import List, Tuple, TypeVar
+import functools
+import inspect
+import sys
 from types import FunctionType
+from typing import Any, Callable, List, Tuple, TypeVar, Union
 
-__VERSION__ = "3.0.0"
+__VERSION__ = "5.0.0"
+
+from overrides.signature import ensure_signature_is_compatible
+
+_WrappedMethod = TypeVar("_WrappedMethod", bound=Union[FunctionType, Callable])
 
 
-_WrappedMethod = TypeVar("_WrappedMethod", bound=FunctionType)
-
-
-def overrides(method: _WrappedMethod) -> _WrappedMethod:
+def overrides(
+    method: _WrappedMethod = None,
+    *,
+    check_signature: bool = True,
+    check_at_runtime: bool = False,
+) -> Any:
     """Decorator to indicate that the decorated method overrides a method in
     superclass.
     The decorator code is executed while loading class. Using this method
@@ -49,22 +57,61 @@ def overrides(method: _WrappedMethod) -> _WrappedMethod:
         def method(self):
             return 1
 
+    :param check_signature: Whether or not to check the signature of the overridden method.
+    :param check_at_runtime: Whether or not to check the overridden method at runtime.
     :raises  AssertionError if no match in super classes for the method name
     :return  method with possibly added (if the method doesn't have one)
         docstring from super class
     """
+    if method:
+        return _overrides(method, check_signature, check_at_runtime)
+    else:
+        return functools.partial(
+            overrides,
+            check_signature=check_signature,
+            check_at_runtime=check_at_runtime,
+        )
+
+
+def _overrides(
+    method: _WrappedMethod,
+    check_signature: bool,
+    check_at_runtime: bool,
+) -> _WrappedMethod:
     setattr(method, "__override__", True)
-    for super_class in _get_base_classes(sys._getframe(2), method.__globals__):
+    for super_class in _get_base_classes(sys._getframe(3), method.__globals__):
         if hasattr(super_class, method.__name__):
-            super_method = getattr(super_class, method.__name__)
-            if hasattr(super_method, "__finalized__"):
-                finalized = getattr(super_method, "__finalized__")
-                if finalized:
-                    raise AssertionError('Method "%s" is finalized' % method.__name__)
-            if not method.__doc__:
-                method.__doc__ = super_method.__doc__
-            return method
-    raise AssertionError('No super class method found for "%s"' % method.__name__)
+            if check_at_runtime:
+
+                @functools.wraps(method)
+                def wrapper(*args, **kwargs):
+                    _validate_method(method, super_class, check_signature)
+                    return method(*args, **kwargs)
+
+                return wrapper  # type: ignore
+            else:
+                _validate_method(method, super_class, check_signature)
+                return method
+    raise TypeError(f"{method.__qualname__}: No super class method found")
+
+
+def _validate_method(method, super_class, check_signature):
+    super_method = getattr(super_class, method.__name__)
+    is_static = isinstance(
+        inspect.getattr_static(super_class, method.__name__), staticmethod
+    )
+    if hasattr(super_method, "__finalized__"):
+        finalized = getattr(super_method, "__finalized__")
+        if finalized:
+            raise TypeError(f"{method.__name__}: is finalized")
+    if not method.__doc__:
+        method.__doc__ = super_method.__doc__
+    if (
+        check_signature
+        and not method.__name__.startswith("__")
+        and not isinstance(super_method, property)
+    ):
+        ensure_signature_is_compatible(super_method, method, is_static)
 
 
 def _get_base_classes(frame, namespace):
@@ -97,7 +144,7 @@ def op_stream(code, max):
 
 
 def _get_base_class_names(frame):
-    """ Get baseclass names from the code object """
+    """Get baseclass names from the code object"""
     co, lasti = frame.f_code, frame.f_lasti
     code = co.co_code
 
@@ -142,5 +189,6 @@ def _get_base_class(components, namespace):
         else:
             obj = getattr(namespace["__builtins__"], components[0])
     for component in components[1:]:
-        obj = getattr(obj, component)
+        if hasattr(obj, component):
+            obj = getattr(obj, component)
     return obj
